<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * format_buttons
 *
 * @package    format_buttons
 * @author     Rodrigo Brandão (rodrigobrandao.com.br)
 * @copyright  2016 Rodrigo Brandão
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

defined('MOODLE_INTERNAL') || die();

require_once($CFG->dirroot.'/course/format/topics/renderer.php');

/**
 * format_buttons_renderer
 *
 */
class format_buttons_renderer extends format_topics_renderer {
    
    /**
     * get_button_section
     *
     */
    protected function get_color_config($name, $course) {
        $return = false;
        if (isset($course->{$name})) {
            $color = str_replace('#', '', $course->{$name});
            $color = substr($color, 0, 6);
            if (is_numeric('0x'.$color)) {
                $return = '#'.$color;
            }
        }
        return $return;
    }

    /**
     * get_button_section
     *
     */
    protected function get_button_section($course, $modinfo, $sectionvisible) {
        global $PAGE;
        $html = '';
        /* css */
        $css = '';
        if ($colorcurrent = $this->get_color_config('colorcurrent', $course)) {
            $css .= '#buttonsectioncontainer .buttonsection.current { background: '.$colorcurrent.'; }';
        }
        if ($colorvisible = $this->get_color_config('colorvisible', $course)) {
             $css .= '#buttonsectioncontainer .buttonsection.sectionvisible { background: '.$colorvisible.'; }';
        }
        if ($css) {
            $html .= html_writer::tag('style', $css);
        }     
        /* html */
        $withoutdivisor = true;
        for ($k = 1; $k <= 4; $k++) {
            if ($course->{'divisor'.$k}) {
                $withoutdivisor = false;
            }
        }
        if ($withoutdivisor) {
            $course->divisor1 = 999;
        }
        $divisorshow = array();
        $count = 1;
        $currentdivisor = 1;
        foreach ($modinfo->get_section_info_all() as $section => $thissection) {
            if ($section == 0) {
                continue;
            }
            if ($section > $course->numsections) {
                continue;
            }
            if (isset($course->{'divisor'.$currentdivisor}) && $count > $course->{'divisor'.$currentdivisor}) {
                $currentdivisor++;
                $count = 1;
            }
            if (isset($course->{'divisor'.$currentdivisor}) && $course->{'divisor'.$currentdivisor} != 0 && !isset($divisorshow[$currentdivisor])) {
                $currentdivisor_html = html_writer::tag('div', str_replace('[br]','<br>', $course->{'divisortext'.$currentdivisor}), array('class' => 'divisortext'));
                $html .= html_writer::tag('div', $currentdivisor_html, array('class' => 'divisorsection'));
                $divisorshow[$currentdivisor] = true;
            }
            /* params */
            $id = 'buttonsection-'.$section;
            $name = (isset($course->{'divisor'.$currentdivisor}) && $course->{'divisor'.$currentdivisor} == 1) ? '&bull;&bull;&bull;' : $count;
            $class = 'buttonsection';
            if ((int)$thissection->visible) {
                $onclick = 'M.format_buttons.show('.$section.')';
            } else {
                $class .= ' sectionhidden';
                $onclick = false;
            }
            if ($course->marker == $section) {
                $class .= ' current';
            }
            if ($sectionvisible == $section) {
                $class .= ' sectionvisible';
            }
            if ($PAGE->user_is_editing()) {
                $onclick = false;
            }
            $html .= html_writer::tag('div', $name, array('id' => $id, 'class' => $class, 'onclick' => $onclick));
            $count++;
        }
        $html = html_writer::tag('div', $html, array('id' => 'buttonsectioncontainer'));
        if ($PAGE->user_is_editing()) {
            $html .= html_writer::tag('div', get_string('editing', 'format_buttons'), array('id' => 'buttonsectionediting'));
        }
        return $html;
    }

    /**
     * start_section_list
     *
     */
    protected function start_section_list($i = 0) {
        return html_writer::start_tag('ul', array('class' => 'buttons'));
    }

    /**
     * section_header
     *
     */
    protected function section_header($section, $course, $onsectionpage, $sectionreturn=null) {
        global $PAGE;
        $o = '';
        $currenttext = '';
        $sectionstyle = '';
        if ($section->section != 0) {
            if (!$section->visible) {
                $sectionstyle = ' hidden';
            } else if (course_get_format($course)->is_section_current($section)) {
                $sectionstyle = ' current';
            }
        }
        $o.= html_writer::start_tag('li', array('id' => 'section-'.$section->section,'class' => 'section main clearfix'.$sectionstyle, 'role'=>'region', 'aria-label'=> get_section_name($course, $section)));
        $o .= html_writer::tag('span', $this->section_title($section, $course), array('class' => 'hidden sectionname'));
        $leftcontent = $this->section_left_content($section, $course, $onsectionpage);
        $o.= html_writer::tag('div', $leftcontent, array('class' => 'left side'));
        $rightcontent = $this->section_right_content($section, $course, $onsectionpage);
        $o.= html_writer::tag('div', $rightcontent, array('class' => 'right side'));
        $o.= html_writer::start_tag('div', array('class' => 'content'));
        $hasnamenotsecpg = (!$onsectionpage && ($section->section != 0 || !is_null($section->name)));
        $hasnamesecpg = ($onsectionpage && ($section->section == 0 && !is_null($section->name)));
        $classes = ' accesshide';
        if ($hasnamenotsecpg || $hasnamesecpg) {
            $classes = '';
        }
        $sectionname = html_writer::tag('span', $this->section_title($section, $course));
        if ($course->showdefaultsectionname) {
            $o.= $this->output->heading($sectionname, 3, 'sectionname' . $classes);
        }
        $o.= html_writer::start_tag('div', array('class' => 'summary'));
        $o.= $this->format_summary_text($section);
        $o.= html_writer::end_tag('div');
        $context = context_course::instance($course->id);
        $o .= $this->section_availability_message($section, has_capability('moodle/course:viewhiddensections', $context));
        return $o;
    }

    /**
     * print_multiple_section_page
     *
     */
    public function print_multiple_section_page($course, $sections, $mods, $modnames, $modnamesused) {
        global $PAGE;
        $modinfo = get_fast_modinfo($course);
        $course = course_get_format($course)->get_course();
        $context = context_course::instance($course->id);
        $completioninfo = new completion_info($course);
        if (isset($_COOKIE['sectionvisible'])) {
            $sectionvisible = $_COOKIE['sectionvisible'];
        } elseif ($course->marker > 0) {
            $sectionvisible = $course->marker;
        } else {
            $sectionvisible = 1;
        }
        $html_section = array();
        foreach ($modinfo->get_section_info_all() as $section => $thissection) {
            $html_section[$section] = '';
            if ($section == 0) {
                $section0 = $thissection;
                continue;
            }
            if ($section > $course->numsections) {
                continue;
            }
            $showsection = $thissection->uservisible or ($thissection->visible and !$thissection->available and !empty($thissection->availableinfo));
            if (!$showsection) {
                if ($thissection->available) {
                    $html_section[$section] .= $this->section_hidden($section, $course->id);
                }
                continue;
            }
            $html_section[$section] .= $this->section_header($thissection, $course, false, 0);
            if ($thissection->uservisible) {
                $html_section[$section] .= $this->courserenderer->course_section_cm_list($course, $thissection, 0);
                $html_section[$section] .= $this->courserenderer->course_section_add_cm_control($course, $section, 0);
            }
            $html_section[$section] .= $this->section_footer();
        }
        /* 0-section is displayed a little different then the others */
        if ($section0->summary or !empty($modinfo->sections[0]) or $PAGE->user_is_editing()) {
            $html_section0 = $this->section_header($section0, $course, false, 0);
            $html_section0 .= $this->courserenderer->course_section_cm_list($course, $section0, 0);
            $html_section0 .= $this->courserenderer->course_section_add_cm_control($course, 0, 0);
            $html_section0 .= $this->section_footer();
        }
        /* echo */
        echo $completioninfo->display_help_icon();
        echo $this->output->heading($this->page_title(), 2, 'accesshide');
        echo $this->course_activity_clipboard($course, 0);
        echo $this->start_section_list();
        if ($course->sectionposition == 0 and isset($html_section0)) {
            echo html_writer::tag('span', $html_section0, array('class' => 'above'));
        }
        echo $this->get_button_section($course, $modinfo, $sectionvisible);
        foreach ($html_section as $current) {
            echo $current;
        }
        if ($course->sectionposition == 1 and isset($html_section0)) {
            echo html_writer::tag('span', $html_section0, array('class' => 'below'));
        }
        if ($PAGE->user_is_editing() and has_capability('moodle/course:update', $context)) {
            foreach ($modinfo->get_section_info_all() as $section => $thissection) {
                if ($section <= $course->numsections or empty($modinfo->sections[$section])) {
                    continue;
                }
                echo $this->stealth_section_header($section);
                echo $this->courserenderer->course_section_cm_list($course, $thissection, 0);
                echo $this->stealth_section_footer();
            }
            echo $this->end_section_list();
            echo html_writer::start_tag('div', array('id' => 'changenumsections', 'class' => 'mdl-right'));
            $straddsection = get_string('increasesections', 'moodle');
            $url = new moodle_url('/course/changenumsections.php', array('courseid' => $course->id, 'increase' => true, 'sesskey' => sesskey()));
            $icon = $this->output->pix_icon('t/switch_plus', $straddsection);
            echo html_writer::link($url, $icon.get_accesshide($straddsection), array('class' => 'increase-sections'));
            if ($course->numsections > 0) {
                $strremovesection = get_string('reducesections', 'moodle');
                $url = new moodle_url('/course/changenumsections.php', array('courseid' => $course->id, 'increase' => false, 'sesskey' => sesskey()));
                $icon = $this->output->pix_icon('t/switch_minus', $strremovesection);
                echo html_writer::link($url, $icon.get_accesshide($strremovesection), array('class' => 'reduce-sections'));
            }
            echo html_writer::end_tag('div');
        } else {
            echo $this->end_section_list();
        }
        /* css */
        echo html_writer::tag('style', '.course-content ul.buttons #section-'.$sectionvisible.' { display: block; }');
        /* javascript */
        if (!$PAGE->user_is_editing()) {
            $PAGE->requires->js_init_call('M.format_buttons.init', array($course->numsections));
        }
    }

}
